package com.agilecontrol.nea.core.cxtab;

import com.agilecontrol.nea.core.control.web.WebUtils;
import com.agilecontrol.nea.core.cxtab.ColumnValueFormatter;
import com.agilecontrol.nea.core.cxtab.CompressCSVFile;
import com.agilecontrol.nea.core.cxtab.EmptyRecordException;
import com.agilecontrol.nea.core.cxtab.FakeColumnFormatter;
import com.agilecontrol.nea.core.cxtab.SQLiteDB;
import com.agilecontrol.nea.core.jcrosstab.jCrosstabResultSet;
import com.agilecontrol.nea.core.jcrosstab.jCrosstabber;
import com.agilecontrol.nea.core.jcrosstab.fun.FunUtil;
import com.agilecontrol.nea.core.query.ColumnLink;
import com.agilecontrol.nea.core.query.Expression;
import com.agilecontrol.nea.core.query.QueryEngine;
import com.agilecontrol.nea.core.query.QueryRequest;
import com.agilecontrol.nea.core.query.QueryRequestImpl;
import com.agilecontrol.nea.core.query.QuerySession;
import com.agilecontrol.nea.core.query.QueryUtils;
import com.agilecontrol.nea.core.report.ReportTools;
import com.agilecontrol.nea.core.schema.Column;
import com.agilecontrol.nea.core.schema.Table;
import com.agilecontrol.nea.core.schema.TableManager;
import com.agilecontrol.nea.core.security.SecurityUtils;
import com.agilecontrol.nea.core.security.User;
import com.agilecontrol.nea.core.util.ConfigValues;
import com.agilecontrol.nea.core.util.MessagesHolder;
import com.agilecontrol.nea.core.velocity.VelocityUtils;
import com.agilecontrol.nea.util.ColumnInterpreter;
import com.agilecontrol.nea.util.CommandExecuter;
import com.agilecontrol.nea.util.Configurations;
import com.agilecontrol.nea.util.NDSException;
import com.agilecontrol.nea.util.NDSRuntimeException;
import com.agilecontrol.nea.util.PairTable;
import com.agilecontrol.nea.util.Tools;
import com.agilecontrol.nea.util.Validator;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFDataFormat;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.HSSFColor.HSSFColorPredefined;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.ss.usermodel.CellType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class CxtabReport {

   private static Logger logger = LoggerFactory.getLogger(CxtabReport.class.getName());
   public static final boolean SKIP_RECORDS_COUNT = "true".equals(WebUtils.getProperty("cxtab.skip.count", "false"));
   public static boolean COMPRESS_CUB_FILE = !"false".equals(WebUtils.getProperty("cxtab.compress", "true"));
   private String cxtabName;
   private int cxtabId = -1;
   private String filterExpr;
   private Expression filterExpression;
   private String filterSQL;
   private String filterDesc;
   private String fileName;
   private String folder;
   private int userId;
   private String fileType;
   private int reportInstanceId = -1;
   private jCrosstabResultSet jxrs;
   private ResultSet rs;
   private Properties props;
   private long startTime = System.currentTimeMillis();
   private Table factTable;
   private String sql;
   private User user;
   private int processInstanceId;
   private ArrayList factDescs;
   private int recordsCount = 0;
   private int dimCount = 0;
   private int meaCount = 0;
   private boolean isToPrinter;
   private String headerCaption;
   private String footerCaption;


   public void setAD_PInstance_ID(int d) {
      this.processInstanceId = d;
   }

   public String create(Connection conn, long startTime) throws Exception {
      this.startTime = startTime;
      Configurations conf = (Configurations)WebUtils.getServletContextManager().getActor("nea.configs");
      String exportRootPath = conf.getProperty("export.root.nds", "/act/home");
      boolean isSystemInDebug = false;
      this.user = SecurityUtils.getUser(this.userId);
      String filePath = Validator.isNull(this.folder)?exportRootPath + File.separator + this.user.getClientDomain() + File.separator + this.user.getId():this.folder;
      File f = new File(filePath);
      if(!f.exists() && !f.mkdirs()) {
         logger.error("Fail to create folder " + filePath);
         throw new NDSException("Could not create folder in server file system. Check server log for detail");
      } else {
         String file;
         QueryRequest query;
         if("xls".equalsIgnoreCase(this.fileType)) {
            query = this.prepareReport(conn, true, false, false, false);
            file = this.fileName + ".xls";
            this.writeCubeFile(filePath + File.separator + file, query, conn);
         } else if("htm".equalsIgnoreCase(this.fileType)) {
            query = this.prepareReport(conn, true, false, true, true);
            file = this.fileName + ".htm";
            this.writeCubeFile(filePath + File.separator + file, query, conn);
         } else if("csv".equalsIgnoreCase(this.fileType)) {
            query = this.prepareCSVReport(conn);
            file = this.fileName + ".csv";
            this.writeCSVFile(filePath + File.separator + file, query, conn);
            if((new File(filePath + File.separator + file)).length() > 10485760L) {
               new CompressCSVFile();
               file = CompressCSVFile.compressCSVFile(filePath + File.separator, file);
            }
         } else if("cus".equalsIgnoreCase(this.fileType)) {
            query = this.prepareReport(conn, true, false, true, true);
            file = this.fileName + ".cus";
            this.writeSQLiteFile(filePath + File.separator + file, query, conn);
         } else {
            query = this.prepareReport(conn, true, false, true, true);
            file = this.fileName + (COMPRESS_CUB_FILE?".cub":".cuz");
            this.writeCubeFile(filePath + File.separator + file, query, conn);
         }

         this.createDescriptionFile(filePath + File.separator + file);
         logRunningTime(this.cxtabId, this.userId, (int)(System.currentTimeMillis() - startTime), this.recordsCount, conn);
         return file;
      }
   }

   private void createDescriptionFile(String file) {
      if(!Validator.isNotNull(this.folder)) {
         try {
            File t = new File(file);
            File descFolder = new File(t.getParent() + File.separator + "desc");
            if(!descFolder.exists()) {
               descFolder.mkdirs();
            }

            String descFile = t.getParent() + File.separator + "desc" + File.separator + t.getName();
            String p;
            if(Validator.isNull(this.filterDesc)) {
               p = "";
            } else {
               p = ":" + this.filterDesc;
            }

            Tools.writeFile(descFile, "[" + this.cxtabName + "]" + p, "UTF-8");
         } catch (Throwable var6) {
            logger.error("Fail to write desc file for " + file + ":" + var6);
         }

      }
   }

   private Column[] getDisplayColumns(QueryRequest req, boolean pk, boolean ak) {
      int[] ids = req.getReportDisplayColumnIndices(pk, ak);
      ArrayList selections = req.getAllSelectionColumnLinks();
      Column[] cols = new Column[ids.length];

      for(int i = 0; i < ids.length; ++i) {
         ColumnLink clink = (ColumnLink)selections.get(ids[i]);
         cols[i] = clink.getLastColumn();
      }

      return cols;
   }

   private HSSFCellStyle getDefaultStyle(HSSFWorkbook wb, boolean isBold) {
      HSSFFont font = wb.createFont();
      font.setFontHeightInPoints((short)10);
      font.setFontName("宋体");
      if(isBold) {
         font.setColor(HSSFColorPredefined.DARK_BLUE.getIndex());
         font.setBold(true);
      }

      HSSFCellStyle style = wb.createCellStyle();
      style.setFont(font);
      return style;
   }

   private void writeExcelFile(String filePath, QueryRequest query, Connection conn) throws Exception {
      (new File(filePath)).delete();
      String template = (String)QueryEngine.getInstance().doQueryOne("select ATTR1 from ad_cxtab where id=" + this.cxtabId, conn);
      logger.debug("reading template xls:" + template);
      File f = template != null?new File(template):null;
      if(f != null && f.exists()) {
         ResultSet rs = null;
         Locale locale = this.user.getLocale();
         if(locale == null) {
            locale = TableManager.getDefaultSysLocale();
         }

         boolean var41 = false;

         try {
            var41 = true;
            rs = conn.createStatement(1003, 1007).executeQuery(this.sql);
            FileInputStream duration = new FileInputStream(template);
            POIFSFileSystem sqlInfo = new POIFSFileSystem(duration);
            HSSFWorkbook wb = new HSSFWorkbook(sqlInfo);
            HSSFSheet sheet = wb.getSheet("data");
            if(sheet == null) {
               throw new NDSException("@template-file-error@");
            }

            this.getDefaultStyle(wb, false);
            HSSFDataFormat format = wb.createDataFormat();
            HSSFCellStyle dateCellStyle = wb.createCellStyle();
            dateCellStyle.setDataFormat(format.getFormat("yyyy-MM-dd"));
            HSSFCellStyle datetimeCellStyle = wb.createCellStyle();
            datetimeCellStyle.setDataFormat(format.getFormat("yyyy-MM-dd hh:mm:ss"));
            short factDescsSize = (short)this.factDescs.size();
            boolean row = false;
            int var49 = 0;
            HSSFRow excel_row = sheet.createRow(var49);
            ArrayList colNames = query.getAllSelectionDescriptions();

            short i;
            HSSFCell cell;
            for(i = 0; i < colNames.size(); ++i) {
               cell = excel_row.createCell(i);
               cell.setCellType(CellType.STRING);
               cell.setCellValue((String)colNames.get(i));
            }

            for(short cols = 0; cols < factDescsSize; ++cols) {
               cell = excel_row.createCell((short)(cols + i));
               cell.setCellType(CellType.STRING);
               cell.setCellValue((String)this.factDescs.get(cols));
            }

            Column[] var50 = this.getDisplayColumns(query, false, false);

            double d;
            String s;
            while(rs.next()) {
               ++var49;
               excel_row = sheet.createRow(var49);

               for(i = 0; i < var50.length; ++i) {
                  cell = excel_row.createCell(i);
                  switch(var50[i].getType()) {
                  case 0:
                     d = rs.getDouble(i + 1);
                     if(!rs.wasNull()) {
                        cell.setCellValue(d);
                     }
                     break;
                  case 1:
                     if(var50[i].getSQLType() == 14) {
                        Timestamp date = rs.getTimestamp(i + 1);
                        if(!rs.wasNull()) {
                           cell.setCellValue(date);
                        }
                     } else {
                        Date var51 = rs.getDate(i + 1);
                        if(!rs.wasNull()) {
                           cell.setCellValue(var51);
                        }
                     }
                     break;
                  case 2:
                     s = rs.getString(i + 1);
                     if(!rs.wasNull()) {
                        cell.setCellType(CellType.STRING);
                        cell.setCellValue(s);
                     }
                     break;
                  case 3:
                     int dn = rs.getInt(i + 1);
                     if(!rs.wasNull()) {
                        try {
                           cell.setCellValue(((SimpleDateFormat)QueryUtils.dateNumberFormatter.get()).parse(String.valueOf(dn)));
                           cell.setCellStyle(dateCellStyle);
                        } catch (Throwable var45) {
                           cell.setCellValue((double)dn);
                        }
                     }
                     break;
                  default:
                     logger.debug("Find at cell(" + var49 + "," + (i + 1) + ") type is invalid");
                  }
               }

               for(short tm = 0; tm < factDescsSize; ++tm) {
                  cell = excel_row.createCell((short)(tm + i));
                  d = rs.getDouble(i + tm + 1);
                  if(!rs.wasNull()) {
                     cell.setCellValue(d);
                  }
               }
            }

            TableManager var52 = TableManager.getInstance();

            for(i = 0; i < var50.length; ++i) {
               Column fileOut = var50[i];
               int colId = fileOut.getId();
               if(fileOut.isValueLimited()) {
                  for(int j = 1; j <= var49; ++j) {
                     try {
                        s = "";
                        cell = sheet.getRow(j).getCell(i);
                        switch(fileOut.getType()) {
                        case 0:
                           d = cell.getNumericCellValue();
                           s = var52.getColumnValueDescription(colId, String.valueOf((int)d), locale);
                           break;
                        case 2:
                           s = cell.getStringCellValue();
                           s = var52.getColumnValueDescription(colId, s, locale);
                           break;
                        default:
                           throw new NDSException("Unexpected column type:" + fileOut.getType() + " for column:" + fileOut);
                        }

                        cell = sheet.getRow(j).createCell(i);
                        cell.setCellType(CellType.STRING);
                        cell.setCellValue(s);
                     } catch (Exception var44) {
                        logger.error("Could not interpret cell(" + j + "," + (i + 1) + "):", var44);
                     }
                  }
               }
            }

            FileOutputStream var53 = new FileOutputStream(filePath);
            wb.write(var53);
            var53.close();
            duration.close();
            sheet = null;
            wb = null;
            var41 = false;
         } finally {
            if(var41) {
               if(rs != null) {
                  try {
                     rs.close();
                  } catch (Throwable var42) {
                     ;
                  }
               }

               int duration1 = (int)((System.currentTimeMillis() - this.startTime) / 1000L);
               String sqlInfo1 = "[" + this.user.getNameWithDomain() + "](" + duration1 + " s) " + this.factTable.getDescription(this.user.getLocale()) + ":" + filePath + " " + this.processInstanceId + ",size:" + (double)f.length() / 1024.0D / 1024.0D + "MB";
               if(duration1 > 10) {
                  logger.info(sqlInfo1);
               } else {
                  logger.debug(sqlInfo1);
               }

            }
         }

         if(rs != null) {
            try {
               rs.close();
            } catch (Throwable var43) {
               ;
            }
         }

         int var47 = (int)((System.currentTimeMillis() - this.startTime) / 1000L);
         String var48 = "[" + this.user.getNameWithDomain() + "](" + var47 + " s) " + this.factTable.getDescription(this.user.getLocale()) + ":" + filePath + " " + this.processInstanceId + ",size:" + (double)f.length() / 1024.0D / 1024.0D + "MB";
         if(var47 > 10) {
            logger.info(var48);
         } else {
            logger.debug(var48);
         }

      } else {
         throw new NDSException("@template-file-not-found@");
      }
   }

   private void writeSQLiteFile(String filePath, QueryRequest query, Connection conn) throws Exception {
      (new File(filePath)).delete();
      QueryEngine.getInstance().executeUpdate("update ad_pinstance_para set p_clob=? where name=\'filter\' and ad_pinstance_id=?", new Object[]{new StringBuilder(this.sql), Integer.valueOf(this.processInstanceId)}, conn);
      QueryEngine.getInstance().executeUpdate("update ad_pinstance_para set info=? where name=\'filename\' and ad_pinstance_id=?", new Object[]{filePath, Integer.valueOf(this.processInstanceId)}, conn);
      SQLiteDB db = new SQLiteDB(this.processInstanceId, this.dimCount, this.sql, filePath, this.cxtabId, this.cxtabName, this.filterDesc, this.user, conn);
      this.recordsCount = db.save();
      File f = new File(filePath);
      if(!f.exists()) {
         logger.error(filePath + " not found in file system");
         throw new NDSException("@cube-creation-failed@:CXB" + this.processInstanceId);
      } else {
         int duration = (int)((System.currentTimeMillis() - this.startTime) / 1000L);
         String sqlInfo = "[" + this.user.getNameWithDomain() + "](" + duration + " s) " + this.factTable.getDescription(this.user.getLocale()) + ":" + filePath + " " + this.processInstanceId + ",size:" + (double)f.length() / 1024.0D / 1024.0D + "MB";
         if(duration > 10) {
            logger.info(sqlInfo);
         } else {
            logger.debug(sqlInfo);
         }

      }
   }

   private void writeCubeFile(String filePath, QueryRequest query, Connection conn) throws Exception {
      (new File(filePath)).delete();
      QueryEngine.getInstance().executeUpdate("update ad_pinstance_para set p_clob=? where name=\'filter\' and ad_pinstance_id=?", new Object[]{new StringBuilder(this.sql), Integer.valueOf(this.processInstanceId)}, conn);
      QueryEngine.getInstance().executeUpdate("update ad_pinstance_para set info=? where name=\'filename\' and ad_pinstance_id=?", new Object[]{filePath, Integer.valueOf(this.processInstanceId)}, conn);
      Configurations conf = (Configurations)WebUtils.getServletContextManager().getActor("nea.configs");
      String pcube = conf.getProperty("program.pcube", "e:/act/bin/pcube.exe");
      String log = conf.getProperty("program.pcube.log", "e:/act/tmp/pcube.log");
      CommandExecuter cmd = new CommandExecuter(log);
      cmd.run(pcube + " " + this.processInstanceId);
      int duration = (int)((System.currentTimeMillis() - this.startTime) / 1000L);
      String sqlInfo = null;
      if(!this.isToPrinter) {
         File f = new File(filePath);
         if(!f.exists()) {
            logger.error(filePath + " not found in file system");
            throw new NDSException("@cube-creation-failed@:CXB" + this.processInstanceId);
         }

         sqlInfo = "[" + this.user.getNameWithDomain() + "](" + duration + " s) " + this.factTable.getDescription(this.user.getLocale()) + ":" + filePath + " " + this.processInstanceId + ",size:" + (double)f.length() / 1024.0D / 1024.0D + "MB";
      } else {
         sqlInfo = "[" + this.user.getNameWithDomain() + "](" + duration + " s) " + this.factTable.getDescription(this.user.getLocale()) + ": Print " + this.headerCaption;
      }

      if(duration > 10) {
         logger.info(sqlInfo);
      } else {
         logger.debug(sqlInfo);
      }

   }

   private void writeCSVFile(String filePath, QueryRequest query, Connection conn) throws Exception {
      (new File(filePath)).delete();
      logger.debug("writeCSVFile:" + this.sql);
      QueryEngine.getInstance().executeUpdate("update ad_pinstance_para set p_clob=? where name=\'filter\' and ad_pinstance_id=?", new Object[]{new StringBuilder(this.sql), Integer.valueOf(this.processInstanceId)}, conn);
      QueryEngine.getInstance().executeUpdate("update ad_pinstance_para set info=? where name=\'filename\' and ad_pinstance_id=?", new Object[]{filePath, Integer.valueOf(this.processInstanceId)}, conn);
      this.rs = conn.createStatement().executeQuery(this.sql);
      FileWriter fw = new FileWriter(filePath, false);
      BufferedWriter outStream = null;
      outStream = new BufferedWriter(fw, 524288);
      if(!ConfigValues.SYSTEMOS.startsWith("windows")) {
         outStream.write(this.getBOM());
      }

      int colcnt = this.rs.getMetaData().getColumnCount();
      ArrayList al = query.getAllSelectionDescriptions();

      int columnTypes;
      for(columnTypes = 0; columnTypes < al.size(); ++columnTypes) {
         outStream.write(al.get(columnTypes) + ",");
      }

      for(columnTypes = 0; columnTypes < this.factDescs.size() - 1; ++columnTypes) {
         outStream.write(this.factDescs.get(columnTypes) + ",");
      }

      outStream.write((String)this.factDescs.get(this.factDescs.size() - 1) + "\r\n");
      int[] var18 = new int[colcnt];

      for(int obj = 1; obj < var18.length; ++obj) {
         var18[obj] = this.rs.getMetaData().getColumnType(obj);
      }

      try {
         while(this.rs.next()) {
            Object var19;
            for(int f = 1; f < colcnt; ++f) {
               var19 = this.rs.getObject(f);
               if(this.rs.wasNull()) {
                  outStream.write(",");
               } else if(var18[f] == 12) {
                  outStream.write("=\"" + var19.toString() + "\",");
               } else {
                  outStream.write(var19.toString() + ",");
               }
            }

            var19 = this.rs.getObject(colcnt);
            if(this.rs.wasNull()) {
               outStream.write("\r\n");
            } else if(this.rs.getMetaData().getColumnType(colcnt) == 12) {
               outStream.write("=\"" + var19.toString() + "\"\r\n");
            } else {
               outStream.write(var19.toString() + "\r\n");
            }
         }
      } finally {
         outStream.flush();
         outStream.close();
         if(this.rs != null) {
            try {
               this.rs.close();
            } catch (Throwable var16) {
               ;
            }
         }

      }

      File var20 = new File(filePath);
      int duration = (int)((System.currentTimeMillis() - this.startTime) / 1000L);
      String sqlInfo = "[" + this.user.getNameWithDomain() + "](" + duration + " s) " + this.factTable.getDescription(this.user.getLocale()) + ":" + filePath + " " + this.processInstanceId + ",size:" + (double)var20.length() / 1024.0D / 1024.0D + "MB";
      if(duration > 10) {
         logger.info(sqlInfo);
      } else {
         logger.debug(sqlInfo);
      }

   }

   private String getDecodeSQL(Column column, int pos) {
      PairTable pt = column.getValues(this.user.getLocale());
      StringBuilder sb = new StringBuilder("DECODE(b");
      sb.append(pos);
      List keys = pt.keyList();

      for(int i = 0; i < keys.size(); ++i) {
         String key = (String)keys.get(i);
         String value = (String)pt.get(key);
         sb.append(",\'").append(key).append("\',\'").append(value).append("\'");
      }

      sb.append(",\'N/A\') b").append(pos);
      return sb.toString();
   }

   private QueryRequest prepareCSVReport(Connection conn) throws Exception {
      QueryEngine engine = QueryEngine.getInstance();
      TableManager manager = TableManager.getInstance();
      if(this.cxtabId == -1) {
         this.cxtabId = Tools.getInt(engine.doQueryOne("select id from ad_cxtab where name=" + QueryUtils.TO_STRING(this.cxtabName) + " and ad_client_id=(select ad_client_id from users where id=" + this.userId + ")", conn), -1);
      }

      List ed = engine.doQueryList("select ad_table_id,name,maxrows from ad_cxtab where id=" + this.cxtabId, conn);
      int factTableId = Tools.getInt(((List)ed.get(0)).get(0), -1);
      String cxtabDesc = (String)((List)ed.get(0)).get(1);
      int maxRows = Tools.getInt(((List)ed.get(0)).get(2), -1);
      this.factTable = manager.getTable(factTableId);
      List dimensionsV = engine.doQueryList("select columnlink, description, measure_order,hidehtml from ad_cxtab_dimension where ad_cxtab_id=" + this.cxtabId + " and isactive=\'Y\' and position_=\'V\' order by orderno asc", conn);
      String cxtabFilter = (String)engine.doQueryOne("select filter from ad_cxtab where id=" + this.cxtabId, conn);
      if(ReportTools.getAvailableSpaceSize(this.user) < 0L) {
         throw new NDSException("@no-free-space-for-user@:" + this.user.getName());
      } else {
         Locale locale = this.user.getLocale();
         logger.debug("Locale for " + this.user.getNameWithDomain() + "(id=" + this.userId + ") is " + locale);
         QuerySession qsession = QueryUtils.createQuerySession(this.userId, this.user.getSecurityGrade(), "", this.user.getLocale());
         QueryRequestImpl query = engine.createRequest(qsession);
         query.setMainTable(factTableId, true, cxtabFilter);
         if(dimensionsV != null && dimensionsV.size() > 0) {
            for(int expr = 0; expr < dimensionsV.size(); ++expr) {
               List sexpr = (List)dimensionsV.get(expr);
               ColumnLink reportInstanceColumnId = new ColumnLink((String)sexpr.get(0));
               query.addSelection(reportInstanceColumnId.getColumnIDs(), false, (String)sexpr.get(1));
            }

            logger.debug("filterExpr=" + this.filterExpr);
            logger.debug("filterSQL=" + this.filterSQL);
            Expression var28 = null;
            if(this.filterExpression == null) {
               if(this.filterExpr != null && !this.filterExpr.trim().equalsIgnoreCase("undefined")) {
                  var28 = Expression.parse(this.filterExpr);
               } else if(this.filterSQL != null) {
                  var28 = new Expression(new ColumnLink(new int[]{this.factTable.getPrimaryKey().getId()}), this.filterSQL, (String)null);
               }
            } else {
               var28 = this.filterExpression;
            }

            logger.debug("expr=" + var28);
            Expression var29 = SecurityUtils.getSecurityFilter(this.factTable.getName(), 1, this.userId, qsession);
            logger.debug("sexpr=" + var29);
            if(var29 != null && !var29.isEmpty()) {
               if(var28 != null && !var28.isEmpty()) {
                  var29 = var28.combine(var29, 1, (String)null);
               }
            } else {
               var29 = var28;
            }

            int var30 = Tools.getInt(engine.doQueryOne("select AD_COLUMN_CXTABINST_ID  from ad_cxtab where id=" + this.cxtabId, conn), -1);
            Column reportInstanceColumn = null;
            if(var30 != -1) {
               reportInstanceColumn = this.factTable.getColumn(var30);
            }

            if(reportInstanceColumn != null && this.reportInstanceId != -1) {
               Expression measures = new Expression(new ColumnLink(new int[]{reportInstanceColumn.getId()}), "=" + this.reportInstanceId, (String)null);
               var29 = measures.combine(var29, 1, (String)null);
            }

            logger.debug("sexpr=" + var29);
            query.addParam(var29);
            List var31 = engine.doQueryList("select ad_column_id, function_, userfact, description,sgrade, VALUEFORMAT,valuename, param1,param2,param2 from ad_cxtab_fact where ad_cxtab_id=" + this.cxtabId + " and isactive=\'Y\' order by orderno asc", conn);
            ArrayList facts = new ArrayList();
            this.factDescs = new ArrayList();
            int userSecurityGrade = this.user.getSecurityGrade();
            boolean mustBeDBGroupBy = false;

            for(int shouldWrap = 0; shouldWrap < var31.size(); ++shouldWrap) {
               List gsql = (List)var31.get(shouldWrap);
               int decodeSQL = Tools.getInt(gsql.get(4), 0);
               if(decodeSQL <= userSecurityGrade) {
                  String pos = (String)gsql.get(2);
                  if(Validator.isNotNull(pos)) {
                     if(pos.startsWith("=")) {
                        facts.add(this.getFunctionSQL(pos, (String)gsql.get(1), var31));
                        this.factDescs.add(gsql.get(3));
                     } else {
                        facts.add(pos);
                        this.factDescs.add(gsql.get(3));
                        mustBeDBGroupBy = true;
                     }
                  } else {
                     String i = (String)gsql.get(1);
                     int dim = Tools.getInt(gsql.get(0), -1);
                     Column cl = TableManager.getInstance().getColumn(dim);
                     if(FunUtil.isValidGroupByFunction(i)) {
                        facts.add(FunUtil.getGroupByFunctionCall(i, cl.isVirtual()?cl.getName():this.factTable.getName() + "." + cl.getName()));
                        this.factDescs.add(gsql.get(3));
                     } else {
                        if(!"AVG".equalsIgnoreCase(i)) {
                           throw new NDSException("不支持此函数:" + i);
                        }

                        if(cl.isVirtual()) {
                           facts.add("AVG(" + cl.getName() + ")");
                        } else {
                           facts.add("AVG(" + this.factTable.getName() + "." + cl.getName() + ")");
                        }

                        this.factDescs.add(gsql.get(3));
                     }
                  }
               }
            }

            if(facts.size() == 0) {
               throw new NDSException("至少配置一个您可以访问的汇总数据字段");
            } else {
               if(!SKIP_RECORDS_COUNT) {
                  String var32 = this.parseVariable(query.toCountSQL());
                  this.recordsCount = Tools.getInt(engine.doQueryOne(var32, conn), -1);
                  if(this.recordsCount == 0) {
                     throw new EmptyRecordException("empty result");
                  }

                  if(maxRows > 0 && this.recordsCount > maxRows) {
                     throw new NDSException("@report-rows-exeed-limit@(" + this.recordsCount + ">" + maxRows + ")");
                  }
               }

               this.sql = query.toGroupBySQL(facts, true);
               this.dimCount = query.getSelectionCount();
               this.meaCount = facts.size();
               boolean var33 = false;
               StringBuilder var34 = new StringBuilder("SELECT ");
               int var36 = 0;
               int var37;
               if(dimensionsV != null && dimensionsV.size() > 0) {
                  for(var37 = 0; var37 < dimensionsV.size(); ++var37) {
                     List var38 = (List)dimensionsV.get(var37);
                     ColumnLink var39 = new ColumnLink((String)var38.get(0));
                     String var35;
                     if(var39.getLastColumn().isValueLimited()) {
                        var33 = true;
                        var35 = this.getDecodeSQL(var39.getLastColumn(), var36);
                     } else {
                        var35 = "b" + var36;
                     }

                     if(var36 > 0) {
                        var34.append(",");
                     }

                     ++var36;
                     var34.append(var35);
                  }
               }

               if(var33) {
                  for(var37 = 0; var37 < facts.size(); ++var37) {
                     var34.append(",s").append(var37);
                  }

                  var34.append(" FROM (").append(this.sql).append(")");
                  this.sql = var34.toString();
               }

               this.sql = this.parseVariable(this.sql);
               logger.debug(this.sql);
               return query;
            }
         } else {
            throw new NDSException("请至少配置一个行定义字段");
         }
      }
   }

   private String getFunctionSQL(String fact, String function, List measures) throws NDSException {
      boolean isSumAvg = "AVG".equalsIgnoreCase(function) && fact.contains("/");
      if(fact.contains("*") && isSumAvg) {
         throw new NDSException("目前不支持在平均函数中定义变量乘法");
      } else {
         String f = fact.substring(1);
         StringBuffer sb = new StringBuffer();
         Pattern pt = Pattern.compile("(\\w+)", 2);
         Matcher m = pt.matcher(f);

         while(m.find()) {
            String v = m.group();
            int startIdx = m.start();
            String colname = null;
            int i = 0;

            while(true) {
               if(i < measures.size()) {
                  List line = (List)measures.get(i);
                  if(!v.equals(line.get(6))) {
                     ++i;
                     continue;
                  }

                  Column col = TableManager.getInstance().getColumn(Tools.getInt(line.get(0), -1));
                  if(col == null) {
                     throw new NDSException("column not found for id=" + line.get(0));
                  }

                  colname = col.getTable().getName() + "." + col.getName();
               }

               if(colname == null) {
                  logger.warn("variable " + v + " could not be found in measure list, fact=" + fact);
                  colname = v;
               }

               if(isSumAvg) {
                  if(startIdx > 0 && f.substring(startIdx - 1, startIdx).equals("/")) {
                     colname = "decode(sum(" + colname + "),0,1,sum(" + colname + "))";
                  } else {
                     colname = "sum(" + colname + ")";
                  }
               }

               m.appendReplacement(sb, colname);
               break;
            }
         }

         m.appendTail(sb);
         String fs;
         if(!isSumAvg) {
            fs = function + "(" + sb.toString() + ")";
         } else {
            fs = sb.toString();
         }

         return fs;
      }
   }

   private QueryRequest prepareReport(Connection conn, boolean sqlOnly, boolean isOnHTML, boolean translateLimitValue, boolean checkUserDims) throws Exception {
      if(!sqlOnly && translateLimitValue) {
         throw new IllegalArgumentException("translateLimitValue only can be true when sqlOnly is true");
      } else {
         QueryEngine engine = QueryEngine.getInstance();
         TableManager manager = TableManager.getInstance();
         if(this.cxtabId == -1) {
            this.cxtabId = Tools.getInt(engine.doQueryOne("select id from ad_cxtab where name=" + QueryUtils.TO_STRING(this.cxtabName) + " and ad_client_id=(select ad_client_id from users where id=" + this.userId + ")", conn), -1);
         }

         List ed = engine.doQueryList("select ad_table_id,name,maxrows,user_dims from ad_cxtab where id=" + this.cxtabId, conn);
         int factTableId = Tools.getInt(((List)ed.get(0)).get(0), -1);
         String cxtabDesc = (String)((List)ed.get(0)).get(1);
         int maxRows = Tools.getInt(((List)ed.get(0)).get(2), -1);
         int userDims = Tools.getInt(((List)ed.get(0)).get(3), -1);
         this.factTable = manager.getTable(factTableId);
         List dimensionsV = engine.doQueryList("select columnlink, description, measure_order,hidehtml,id from ad_cxtab_dimension where ad_cxtab_id=" + this.cxtabId + " and isactive=\'Y\' and position_=\'V\' " + (isOnHTML?"and hidehtml=\'N\'":"") + " order by orderno asc", conn);
         List dimensionsH = engine.doQueryList("select columnlink, description, measure_order, hidehtml,id from ad_cxtab_dimension where ad_cxtab_id=" + this.cxtabId + " and isactive=\'Y\' and position_=\'H\' " + (isOnHTML?"and hidehtml=\'N\'":"") + " order by orderno asc", conn);
         List dimensionsP = engine.doQueryList("select columnlink, description, measure_order, hidehtml,id from ad_cxtab_dimension where ad_cxtab_id=" + this.cxtabId + " and isactive=\'Y\' and position_=\'P\' " + (isOnHTML?"and hidehtml=\'N\'":"") + " order by orderno asc", conn);
         boolean shouldCheckUserDims = checkUserDims && userDims > 0 && userDims < dimensionsV.size() + dimensionsH.size() + dimensionsP.size();
         String cxtabFilter;
         int expr;
         if(shouldCheckUserDims) {
            cxtabFilter = (String)engine.doQueryOne("select value from ad_user_pref where ad_user_id=? and module=? and name=?", new Object[]{Integer.valueOf(this.userId), "cxtabdim", String.valueOf(this.cxtabId)}, conn);
            if(!Validator.isNotNull(cxtabFilter)) {
               logger.error("Internal Error: user work on a cxtab which has more dims than limitation, cxtab id=" + this.cxtabId);
               throw new NDSException("当前报表维度超过了限制");
            }

            String[] locale = cxtabFilter.split(",");
            HashSet qsession = new HashSet();
            String[] query = locale;
            expr = locale.length;

            for(int sexpr = 0; sexpr < expr; ++sexpr) {
               String reportInstanceColumnId = query[sexpr];
               int reportInstanceColumn = Tools.getInt(reportInstanceColumnId, -1);
               if(reportInstanceColumn != -1) {
                  qsession.add(Integer.valueOf(reportInstanceColumn));
               }
            }

            this.removeDims(dimensionsV, qsession);
            this.removeDims(dimensionsH, qsession);
            this.removeDims(dimensionsP, qsession);
         }

         cxtabFilter = (String)engine.doQueryOne("select filter from ad_cxtab where id=" + this.cxtabId, conn);
         if(ReportTools.getAvailableSpaceSize(this.user) < 0L) {
            throw new NDSException("@no-free-space-for-user@:" + this.user.getName());
         } else {
            Locale var51 = this.user.getLocale();
            logger.debug("Locale for " + this.user.getNameWithDomain() + "(id=" + this.userId + ") is " + var51);
            QuerySession var52 = QueryUtils.createQuerySession(this.userId, this.user.getSecurityGrade(), "", this.user.getLocale());
            QueryRequestImpl var53 = engine.createRequest(var52);
            var53.setMainTable(factTableId, true, cxtabFilter);
            List var54;
            ColumnLink var55;
            if(dimensionsV != null && dimensionsV.size() > 0) {
               for(expr = 0; expr < dimensionsV.size(); ++expr) {
                  var54 = (List)dimensionsV.get(expr);
                  var55 = new ColumnLink((String)var54.get(0));
                  var53.addSelection(var55.getColumnIDs(), false, (String)var54.get(1));
               }
            } else if(!sqlOnly) {
               var53.addSelection("1", "");
            }

            if(dimensionsH != null && dimensionsH.size() > 0) {
               for(expr = 0; expr < dimensionsH.size(); ++expr) {
                  var54 = (List)dimensionsH.get(expr);
                  var55 = new ColumnLink((String)var54.get(0));
                  var53.addSelection(var55.getColumnIDs(), false, (String)var54.get(1));
               }
            } else if(!sqlOnly) {
               var53.addSelection("1", "");
            }

            if(!isOnHTML && dimensionsP != null && dimensionsP.size() > 0) {
               for(expr = 0; expr < dimensionsP.size(); ++expr) {
                  var54 = (List)dimensionsP.get(expr);
                  var55 = new ColumnLink((String)var54.get(0));
                  var53.addSelection(var55.getColumnIDs(), false, (String)var54.get(1));
               }
            }

            logger.debug("filterExpr=" + this.filterExpr);
            logger.debug("filterSQL=" + this.filterSQL);
            Expression var56 = null;
            if(this.filterExpression == null) {
               if(this.filterExpr != null && !this.filterExpr.trim().equalsIgnoreCase("undefined")) {
                  var56 = Expression.parse(this.filterExpr);
               } else if(this.filterSQL != null) {
                  var56 = new Expression(new ColumnLink(new int[]{this.factTable.getPrimaryKey().getId()}), this.filterSQL, (String)null);
               }
            } else {
               var56 = this.filterExpression;
            }

            logger.debug("expr=" + var56);
            Expression var57 = SecurityUtils.getSecurityFilter(this.factTable.getName(), 1, this.userId, var52);
            logger.debug("sexpr=" + var57);
            if(var57 != null && !var57.isEmpty()) {
               if(var56 != null && !var56.isEmpty()) {
                  var57 = var56.combine(var57, 1, (String)null);
               }
            } else {
               var57 = var56;
            }

            int var58 = Tools.getInt(engine.doQueryOne("select AD_COLUMN_CXTABINST_ID  from ad_cxtab where id=" + this.cxtabId, conn), -1);
            Column var59 = null;
            if(var58 != -1) {
               var59 = this.factTable.getColumn(var58);
            }

            if(var59 != null && this.reportInstanceId != -1) {
               Expression measures = new Expression(new ColumnLink(new int[]{var59.getId()}), "=" + this.reportInstanceId, (String)null);
               var57 = measures.combine(var57, 1, (String)null);
            }

            logger.debug("sexpr=" + var57);
            var53.addParam(var57);
            List var60 = engine.doQueryList("select ad_column_id, function_, userfact, description,sgrade, VALUEFORMAT,valuename, param1,param2,param2 from ad_cxtab_fact where ad_cxtab_id=" + this.cxtabId + " and isactive=\'Y\' order by orderno asc", conn);
            ArrayList facts = new ArrayList();
            this.factDescs = new ArrayList();
            int userSecurityGrade = this.user.getSecurityGrade();
            boolean isDBGroupByEnabled = true;
            boolean mustBeDBGroupBy = false;

            int jx;
            List pos;
            int ft;
            int dimensionsHDesc;
            for(jx = 0; jx < var60.size(); ++jx) {
               pos = (List)var60.get(jx);
               ft = Tools.getInt(pos.get(4), 0);
               if(ft <= userSecurityGrade) {
                  String ci = (String)pos.get(2);
                  if(Validator.isNotNull(ci)) {
                     if(!ci.startsWith("=")) {
                        facts.add(ci);
                        this.factDescs.add(pos.get(3));
                        mustBeDBGroupBy = true;
                     }
                  } else {
                     String dimensionsVDesc = (String)pos.get(1);
                     dimensionsHDesc = Tools.getInt(pos.get(0), -1);
                     Column factDesc = TableManager.getInstance().getColumn(dimensionsHDesc);
                     if(FunUtil.isValidGroupByFunction(dimensionsVDesc)) {
                        facts.add(FunUtil.getGroupByFunctionCall(dimensionsVDesc, factDesc.isVirtual()?factDesc.getName():this.factTable.getName() + "." + factDesc.getName()));
                        this.factDescs.add(pos.get(3));
                     } else {
                        isDBGroupByEnabled = false;
                     }
                  }
               }
            }

            if(!SKIP_RECORDS_COUNT) {
               String var61 = this.parseVariable(var53.toCountSQL());
               this.recordsCount = Tools.getInt(engine.doQueryOne(var61, conn), -1);
               if(this.recordsCount == 0) {
                  throw new EmptyRecordException("empty result");
               }

               if(maxRows > 0 && this.recordsCount > maxRows) {
                  throw new NDSException("@report-rows-exeed-limit@(" + this.recordsCount + ">" + maxRows + ")");
               }
            }

            int var66;
            if(isDBGroupByEnabled) {
               if(facts.size() == 0) {
                  throw new NDSException("No fact valid for current report, check sum fields and their security grade");
               } else {
                  this.sql = var53.toGroupBySQL(facts, true);
                  this.dimCount = var53.getSelectionCount();
                  this.meaCount = facts.size();
                  if(sqlOnly) {
                     if(translateLimitValue) {
                        boolean var64 = false;
                        StringBuilder var70 = new StringBuilder("SELECT ");
                        var66 = 0;
                        String var71;
                        int var72;
                        List var75;
                        ColumnLink var80;
                        if(dimensionsV != null && dimensionsV.size() > 0) {
                           for(var72 = 0; var72 < dimensionsV.size(); ++var72) {
                              var75 = (List)dimensionsV.get(var72);
                              var80 = new ColumnLink((String)var75.get(0));
                              if(var80.getLastColumn().isValueLimited()) {
                                 var64 = true;
                                 var71 = this.getDecodeSQL(var80.getLastColumn(), var66);
                              } else {
                                 var71 = "b" + var66;
                              }

                              if(var66 > 0) {
                                 var70.append(",");
                              }

                              ++var66;
                              var70.append(var71);
                           }
                        }

                        if(dimensionsH != null && dimensionsH.size() > 0) {
                           for(var72 = 0; var72 < dimensionsH.size(); ++var72) {
                              var75 = (List)dimensionsH.get(var72);
                              var80 = new ColumnLink((String)var75.get(0));
                              if(var80.getLastColumn().isValueLimited()) {
                                 var64 = true;
                                 var71 = this.getDecodeSQL(var80.getLastColumn(), var66);
                              } else {
                                 var71 = "b" + var66;
                              }

                              if(var66 > 0) {
                                 var70.append(",");
                              }

                              ++var66;
                              var70.append(var71);
                           }
                        }

                        if(dimensionsP != null && dimensionsP.size() > 0) {
                           for(var72 = 0; var72 < dimensionsP.size(); ++var72) {
                              var75 = (List)dimensionsP.get(var72);
                              var80 = new ColumnLink((String)var75.get(0));
                              if(var80.getLastColumn().isValueLimited()) {
                                 var64 = true;
                                 var71 = this.getDecodeSQL(var80.getLastColumn(), var66);
                              } else {
                                 var71 = "b" + var66;
                              }

                              if(var66 > 0) {
                                 var70.append(",");
                              }

                              ++var66;
                              var70.append(var71);
                           }
                        }

                        if(var64) {
                           if(isDBGroupByEnabled) {
                              for(var72 = 0; var72 < facts.size(); ++var72) {
                                 var70.append(",s").append(var72);
                              }

                              var70.append(" FROM (").append(this.sql).append(")");
                              this.sql = var70.toString();
                           } else {
                              for(var72 = 0; var72 < var60.size(); ++var72) {
                                 var70.append(",b").append(var66);
                                 ++var66;
                              }

                              var70.append(" FROM (").append(this.sql).append(")");
                              this.sql = var70.toString();
                           }
                        }
                     }

                     this.sql = this.parseVariable(this.sql);
                     logger.debug(this.sql);
                     return var53;
                  } else {
                     jCrosstabber var62 = new jCrosstabber();
                     int var63 = 1;
                     StringBuilder var69 = new StringBuilder();
                     ColumnLink file;
                     String rs;
                     ColumnValueFormatter var65;
                     ColumnInterpreter var67;
                     if(dimensionsV != null && dimensionsV.size() > 0) {
                        for(dimensionsHDesc = 0; dimensionsHDesc < dimensionsV.size(); ++dimensionsHDesc) {
                           List var74 = (List)dimensionsV.get(dimensionsHDesc);
                           file = new ColumnLink((String)var74.get(0));
                           var67 = manager.getColumnInterpreter(file.getLastColumn().getId());
                           if(var67 != null) {
                              var65 = new ColumnValueFormatter(var67, var51);
                           } else {
                              var65 = null;
                           }

                           rs = (String)var74.get(1);
                           if(Validator.isNull(rs)) {
                              rs = file.getDescription(var51);
                           }

                           var62.addVerticalSliceByTableColumnIndex(var63, var65, rs);
                           ++var63;
                           if(dimensionsHDesc > 0) {
                              var69.append(",");
                           }

                           var69.append(rs);
                        }
                     } else {
                        var62.addVerticalSliceByTableColumnIndex(var63, FakeColumnFormatter.INSTANCE, "");
                        ++var63;
                     }

                     StringBuilder var73 = new StringBuilder();
                     String duration;
                     if(dimensionsH != null && dimensionsH.size() > 0) {
                        for(int var76 = 0; var76 < dimensionsH.size(); ++var76) {
                           List var78 = (List)dimensionsH.get(var76);
                           ColumnLink var81 = new ColumnLink((String)var78.get(0));
                           var67 = manager.getColumnInterpreter(var81.getLastColumn().getId());
                           if(var67 != null) {
                              var65 = new ColumnValueFormatter(var67, var51);
                           } else {
                              var65 = null;
                           }

                           duration = (String)var78.get(1);
                           if(Validator.isNull(duration)) {
                              duration = var81.getDescription(var51);
                           }

                           var62.addHorizontalSliceByTableColumnIndex(var63, var65, duration);
                           ++var63;
                           if(var76 > 0) {
                              var73.append(",");
                           }

                           var73.append(duration);
                        }
                     } else {
                        var62.addHorizontalSliceByTableColumnIndex(var63, FakeColumnFormatter.INSTANCE, "");
                        ++var63;
                     }

                     StringBuilder var77 = new StringBuilder();

                     String sqlInfo;
                     for(int var79 = 0; var79 < var60.size(); ++var79) {
                        rs = null;
                        duration = (String)((List)var60.get(var79)).get(1);
                        if(isDBGroupByEnabled && "COUNT".equalsIgnoreCase(duration)) {
                           duration = "SUM";
                        }

                        sqlInfo = (String)((List)var60.get(var79)).get(2);
                        var62.addDataRowsColumnByIndex(var63, duration, rs, (String)((List)var60.get(var79)).get(3), (String)((List)var60.get(var79)).get(4), sqlInfo, (String)((List)var60.get(var79)).get(5));
                        if(var79 > 0) {
                           var77.append(",");
                        }

                        var77.append((String)((List)var60.get(var79)).get(3));
                        if(sqlInfo == null || !sqlInfo.startsWith("=")) {
                           ++var63;
                        }
                     }

                     file = null;
                     ResultSet var82 = null;
                     boolean var47 = false;

                     try {
                        var47 = true;
                        var82 = conn.createStatement(1004, 1007).executeQuery(this.sql);
                        this.jxrs = var62.getCrosstabResultSet(var82);
                        this.props = new Properties();
                        duration = var53.getParamDesc(true);
                        this.props.setProperty("title", this.factTable.getDescription(this.user.getLocale()) + " - " + duration + " - " + MessagesHolder.getInstance().getMessage(this.user.getLocale(), "cxtabreport"));
                        this.props.setProperty("subject", MessagesHolder.getInstance().getMessage(this.user.getLocale(), "cxtabreport") + " - " + cxtabDesc);
                        this.props.setProperty("facttable", this.factTable.getDescription(this.user.getLocale()));
                        this.props.setProperty("creator", MessagesHolder.getInstance().getMessage(this.user.getLocale(), "creator") + " : " + this.user.getName());
                        this.props.setProperty("creationdate", MessagesHolder.getInstance().getMessage(this.user.getLocale(), "creationdate") + " : " + ((SimpleDateFormat)QueryUtils.dateTimeSecondsFormatter.get()).format(new java.util.Date()));
                        this.props.setProperty("description", MessagesHolder.getInstance().getMessage(this.user.getLocale(), "filter-description") + " : " + duration);
                        this.props.setProperty("axis_h", MessagesHolder.getInstance().getMessage(this.user.getLocale(), "axis-h") + " : " + var73.toString());
                        this.props.setProperty("axis_v", MessagesHolder.getInstance().getMessage(this.user.getLocale(), "axis-v") + " : " + var69.toString());
                        this.props.setProperty("facts", MessagesHolder.getInstance().getMessage(this.user.getLocale(), "fact-desc") + " : " + var77.toString());
                        this.props.put("startfrom", new Long(this.startTime));
                        this.props.put("rowsfetched", new Long(this.jxrs.getRowsFetched()));
                        this.props.setProperty("sql", this.sql);
                        var47 = false;
                     } finally {
                        if(var47) {
                           if(var82 != null) {
                              try {
                                 var82.close();
                              } catch (Throwable var48) {
                                 ;
                              }
                           }

                           int duration1 = (int)((System.currentTimeMillis() - this.startTime) / 1000L);
                           String sqlInfo1 = "[" + this.user.getNameWithDomain() + "](" + duration1 + " s) " + this.factTable.getDescription(this.user.getLocale()) + ":" + this.sql;
                           if(duration1 > 10) {
                              logger.info(sqlInfo1);
                           } else {
                              logger.debug(sqlInfo1);
                           }

                        }
                     }

                     if(var82 != null) {
                        try {
                           var82.close();
                        } catch (Throwable var49) {
                           ;
                        }
                     }

                     int var83 = (int)((System.currentTimeMillis() - this.startTime) / 1000L);
                     sqlInfo = "[" + this.user.getNameWithDomain() + "](" + var83 + " s) " + this.factTable.getDescription(this.user.getLocale()) + ":" + this.sql;
                     if(var83 > 10) {
                        logger.info(sqlInfo);
                     } else {
                        logger.debug(sqlInfo);
                     }

                     return var53;
                  }
               }
            } else if(mustBeDBGroupBy) {
               throw new NDSException("Cxtab configuration error, found user fact(db group by function) and invalid db group by function (e.g. avg) in the same time");
            } else {
               this.dimCount = var53.getSelectionCount();
               this.meaCount = var60.size();

               for(jx = 0; jx < var60.size(); ++jx) {
                  pos = (List)var60.get(jx);
                  ft = Tools.getInt(pos.get(4), 0);
                  if(ft <= userSecurityGrade) {
                     var66 = Tools.getInt(pos.get(0), -1);
                     Column var68 = TableManager.getInstance().getColumn(var66);
                     if(var68 != null) {
                        var53.addSelection(var66);
                     } else {
                        var53.addSelection("1", "1");
                     }
                  }
               }

               this.sql = var53.toSQL();
               throw new NDSException("Unexpected sql handled, no group by clause!");
            }
         }
      }
   }

   private void removeDims(List dims, HashSet validDimIds) {
      for(int i = dims.size() - 1; i >= 0; --i) {
         int id = Tools.getInt(((List)dims.get(i)).get(4), -1);
         if(!validDimIds.contains(Integer.valueOf(id))) {
            dims.remove(i);
         }
      }

   }

   public int getReportInstanceId() {
      return this.reportInstanceId;
   }

   public void setReportInstanceId(int id) {
      this.reportInstanceId = id;
   }

   public int getUserId() {
      return this.userId;
   }

   public String getFileType() {
      return this.fileType;
   }

   public void setFileType(String fileType) {
      this.fileType = fileType;
   }

   public void setUserId(int userId) {
      this.userId = userId;
   }

   public String getCxtabName() {
      return this.cxtabName;
   }

   public String getFileName() {
      return this.fileName;
   }

   public String getFilterExpr() {
      return this.filterExpr;
   }

   public String getFilterSQL() {
      return this.filterSQL;
   }

   public void setCxtabName(String CxtabName) {
      this.cxtabName = CxtabName;
   }

   public void setFileName(String fileName) {
      try {
         this.fileName = this.parseVariable(fileName);
      } catch (Exception var3) {
         logger.error("fail to parse fileName:" + fileName, var3);
         throw new NDSRuntimeException("Fail to parse file name " + fileName + ":" + fileName);
      }
   }

   public void setFilterExpr(String filterExpr) {
      this.filterExpr = filterExpr;
   }

   public void setFilterSQL(String filterSQL) {
      this.filterSQL = filterSQL;
   }

   public int getCxtabId() {
      return this.cxtabId;
   }

   public void setCxtabId(int cxtabId) {
      this.cxtabId = cxtabId;
   }

   public Expression getFilterExpression() {
      return this.filterExpression;
   }

   public void setFilterExpression(Expression filterExpression) {
      this.filterExpression = filterExpression;
   }

   public void setFilterDesc(String s) {
      this.filterDesc = s;
   }

   public String getFolder() {
      return this.folder;
   }

   public void setFolder(String folder) {
      this.folder = folder;
   }

   public static void logRunningTime(int cxtabId, int userId, int duration, int recordCnt, Connection conn) throws Exception {
      int seconds = duration / 1000;
      PreparedStatement pstmtm = conn.prepareStatement("merge into ad_cxtab_stat g using(select ? ad_cxtab_id, id user_ID, ad_client_id,ad_org_id from users where id=?  )b on(b.ad_cxtab_id=g.ad_cxtab_id and b.user_id=g.user_id) when matched then update set g.cnt=g.cnt+1, g.tot_time=g.tot_time+?,g.tot_rows=g.tot_rows+?,g.max_time=GREATEST(g.max_time,?), g.max_rows=GREATEST(g.max_rows,?), last_duration=?,last_rows=?, last_time=sysdate  when not matched then insert (g.id, g.ad_client_id, g.ad_org_id, g.user_id,g.ad_cxtab_id,cnt,tot_time,tot_rows,max_time,max_rows,last_duration,last_rows,last_time) values( get_sequences(\'ad_cxtab_stat\'), b.ad_client_id, b.ad_org_id, b.user_ID,  b.ad_cxtab_id, 1, ?,?,?,?,?,?,sysdate)");

      try {
         byte i = 0;
         int var15 = i + 1;
         pstmtm.setInt(var15, cxtabId);
         ++var15;
         pstmtm.setInt(var15, userId);
         ++var15;
         pstmtm.setInt(var15, seconds);
         ++var15;
         pstmtm.setInt(var15, recordCnt);
         ++var15;
         pstmtm.setInt(var15, seconds);
         ++var15;
         pstmtm.setInt(var15, recordCnt);
         ++var15;
         pstmtm.setInt(var15, seconds);
         ++var15;
         pstmtm.setInt(var15, recordCnt);
         ++var15;
         pstmtm.setInt(var15, seconds);
         ++var15;
         pstmtm.setInt(var15, recordCnt);
         ++var15;
         pstmtm.setInt(var15, seconds);
         ++var15;
         pstmtm.setInt(var15, recordCnt);
         ++var15;
         pstmtm.setInt(var15, seconds);
         ++var15;
         pstmtm.setInt(var15, recordCnt);
         pstmtm.executeUpdate();
      } finally {
         try {
            pstmtm.close();
         } catch (Throwable var13) {
            ;
         }

      }

   }

   private String parseVariable(String sql) throws Exception {
      if(sql.indexOf(36) > 0) {
         sql = VelocityUtils.evaluate(sql);
      }

      return sql;
   }

   public boolean isToPrinter() {
      return this.isToPrinter;
   }

   public void setToPrinter(boolean isToPrinter) {
      this.isToPrinter = isToPrinter;
   }

   public String getHeaderCaption() {
      return this.headerCaption;
   }

   public void setHeaderCaption(String headerCaption) {
      this.headerCaption = headerCaption;
   }

   public String getFooterCaption() {
      return this.footerCaption;
   }

   public void setFooterCaption(String footerCaption) {
      this.footerCaption = footerCaption;
   }

   public String getBOM() {
      byte[] b = new byte[]{(byte)-17, (byte)-69, (byte)-65};
      return new String(b);
   }

}
